@@ -67,7 +67,7 @@ static int     __putindent(FILE *fp, struct message *mp, int maxwidth);
 static int     _dispc(struct message *mp, char const *a);
 
 /* Shared `z' implementation */
-static int     _scroll1(char *arg, int onlynew);
+static int a_cmd_scroll(char const *arg, bool_t onlynew);
 
 /* Shared `headers' implementation */
 static int     _headers(int msgspec);
@@ -681,48 +681,83 @@ _dispc(struct message *mp, char const *a)
 }
 
 static int
-_scroll1(char *arg, int onlynew)
-{
-   int msgspec, size;
-   NYD_ENTER;
+a_cmd_scroll(char const *arg, bool_t onlynew){
+   long l;
+   char *eptr;
+   bool_t isabs;
+   int msgspec, size, maxs;
+   NYD2_ENTER;
 
+   /* TODO scroll problem: we do not know whether + and $ have already reached
+    * TODO the last screen in threaded mode */
    msgspec = onlynew ? -1 : 0;
    size = screensize();
+   if((maxs = msgCount / size) > 0 && msgCount % size == 0)
+      --maxs;
 
-   if (arg[0] != '\0' && arg[1] != '\0')
-      goto jerr;
-   switch (*arg) {
-   case '1': case '2': case '3': case '4': case '5':
-   case '6': case '7': case '8': case '9': case '0':
-      _screen = atoi(arg);
-      goto jscroll_forward;
+   switch(*arg){
    case '\0':
       ++_screen;
-      goto jscroll_forward;
+      goto jfwd;
+   case '^':
+      if(arg[1] != '\0')
+         goto jerr;
+      if(_screen == 0)
+         goto jerrbwd;
+      _screen = 0;
+      break;
    case '$':
-      _screen = msgCount / size;
-      goto jscroll_forward;
+      if(arg[1] != '\0')
+         goto jerr;
+      if(_screen == maxs)
+         goto jerrfwd;
+      _screen = maxs;
+      break;
    case '+':
-      if (arg[1] == '\0')
+      if(arg[1] == '\0')
          ++_screen;
-      else
-         _screen += atoi(arg + 1);
-jscroll_forward:
-      if (_screen * size > msgCount) {
-         _screen = msgCount / size;
+      else{
+         isabs = FAL0;
+
+         ++arg;
+         if(0){
+   case '1': case '2': case '3': case '4': case '5':
+   case '6': case '7': case '8': case '9': case '0':
+            isabs = TRU1;
+         }
+         l = strtol(arg, &eptr, 10);
+         if(*eptr != '\0')
+            goto jerr;
+         if(l > maxs - (isabs ? 0 : _screen))
+            goto jerrfwd;
+         _screen = isabs ? (int)l : _screen + l;
+      }
+jfwd:
+      if(_screen > maxs){
+jerrfwd:
+         _screen = maxs;
          printf(_("On last screenful of messages\n"));
       }
       break;
+
    case '-':
-      if (arg[1] == '\0')
+      if(arg[1] == '\0')
          --_screen;
-      else
-         _screen -= atoi(arg + 1);
-      if (_screen < 0) {
+      else{
+         ++arg;
+         l = strtol(arg, &eptr, 10);
+         if(*eptr != '\0')
+            goto jerr;
+         if(l > _screen)
+            goto jerrbwd;
+         _screen -= l;
+      }
+      if(_screen < 0){
+jerrbwd:
          _screen = 0;
          printf(_("On first screenful of messages\n"));
       }
-      if (msgspec == -1)
+      if(msgspec == -1)
          msgspec = -2;
       break;
    default:
@@ -734,7 +769,7 @@ jerr:
 
    size = _headers(msgspec);
 jleave:
-   NYD_LEAVE;
+   NYD2_LEAVE;
    return size;
 }
 
@@ -1071,7 +1106,7 @@ c_scroll(void *v)
    int rv;
    NYD_ENTER;
 
-   rv = _scroll1(v, 0);
+   rv = a_cmd_scroll(v, FAL0);
    NYD_LEAVE;
    return rv;
 }
@@ -1082,7 +1117,7 @@ c_Scroll(void *v)
    int rv;
    NYD_ENTER;
 
-   rv = _scroll1(v, 1);
+   rv = a_cmd_scroll(v, TRU1);
    NYD_LEAVE;
    return rv;
 }
@@ -1368,13 +1403,16 @@ c_folders(void *v)
    if (which_protocol(name) == PROTO_IMAP) {
 #ifdef HAVE_IMAP
       imap_folders(name, *argv == NULL);
+      rv = 0;
 #else
       rv = c_cmdnotsupp(NULL);
 #endif
    } else {
       if ((cmd = ok_vlook(LISTER)) == NULL)
          cmd = XLISTER;
-      run_command(cmd, 0, -1, -1, name, NULL, NULL);
+      rv = run_command(cmd, 0, -1, -1, name, NULL, NULL);
+      if (rv < 0)
+         rv = 1;
    }
 jleave:
    NYD_LEAVE;
